"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloCacheVisitor = void 0;
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const auto_bind_1 = __importDefault(require("auto-bind"));
class ApolloCacheVisitor extends visitor_plugin_common_1.ClientSideBaseVisitor {
    constructor(schema, fragments, rawConfig, documents) {
        super(schema, fragments, rawConfig, {
            withHooks: visitor_plugin_common_1.getConfigValue(rawConfig.withHooks, false),
            apolloReactCommonImportFrom: visitor_plugin_common_1.getConfigValue(rawConfig.apolloReactCommonImportFrom, rawConfig.reactApolloVersion === 3
                ? "@apollo/client"
                : "@apollo/react-common"),
            apolloReactHooksImportFrom: visitor_plugin_common_1.getConfigValue(rawConfig.apolloReactHooksImportFrom, rawConfig.reactApolloVersion === 3
                ? "@apollo/client"
                : "@apollo/react-hooks"),
            apolloImportFrom: visitor_plugin_common_1.getConfigValue(rawConfig.apolloImportFrom, rawConfig.reactApolloVersion === 3 ? "@apollo/client" : "apollo-client"),
            apolloCacheImportFrom: visitor_plugin_common_1.getConfigValue(rawConfig.apolloCacheImportFrom, rawConfig.reactApolloVersion === 3
                ? "@apollo/client"
                : "apollo-cache-inmemory"),
            reactApolloVersion: visitor_plugin_common_1.getConfigValue(rawConfig.reactApolloVersion, 2),
            excludePatterns: visitor_plugin_common_1.getConfigValue(rawConfig.excludePatterns, null),
            excludePatternsOptions: visitor_plugin_common_1.getConfigValue(rawConfig.excludePatternsOptions, ""),
            dataIdFromObjectImport: visitor_plugin_common_1.getConfigValue(rawConfig.dataIdFromObjectImport, null),
            dataIdFromObjectName: visitor_plugin_common_1.getConfigValue(rawConfig.dataIdFromObjectName, "defaultDataIdFromObject"),
            generateFragmentsRead: visitor_plugin_common_1.getConfigValue(rawConfig.generateFragmentsRead, true),
            generateFragmentsWrite: visitor_plugin_common_1.getConfigValue(rawConfig.generateFragmentsWrite, true),
            generateQueriesRead: visitor_plugin_common_1.getConfigValue(rawConfig.generateQueriesRead, true),
            generateQueriesWrite: visitor_plugin_common_1.getConfigValue(rawConfig.generateQueriesWrite, true),
            pre: visitor_plugin_common_1.getConfigValue(rawConfig.pre, ""),
            post: visitor_plugin_common_1.getConfigValue(rawConfig.post, ""),
            customImports: visitor_plugin_common_1.getConfigValue(rawConfig.customImports, null),
        });
        this.imports = new Set();
        this._externalImportPrefix = this.config.importOperationTypesFrom
            ? `${this.config.importOperationTypesFrom}.`
            : "";
        this._documents = documents;
        auto_bind_1.default(this);
    }
    getImports() {
        this.imports.add(`import { ApolloClient } from '${this.config.apolloImportFrom}';`);
        this.imports.add(`import { defaultDataIdFromObject, NormalizedCacheObject } from '${this.config.apolloCacheImportFrom}';`);
        if (this.config.dataIdFromObjectImport) {
            this.imports.add(`import { ${this.config.dataIdFromObjectName} } from '${this.config.dataIdFromObjectImport}';`);
        }
        if (this.config.customImports) {
            this.imports.add(this.config.customImports);
        }
        const baseImports = super.getImports();
        const hasOperations = this._collectedOperations.length > 0;
        if (!hasOperations) {
            return baseImports;
        }
        return [...baseImports, ...Array.from(this.imports)];
    }
    _buildOperationReadCache(node, documentVariableName, operationResultType, operationVariablesTypes) {
        if (node.operation === "mutation") {
            return "";
        }
        const operationName = this.convertName(node.name.value, {
            useTypesPrefix: false,
        });
        if (this.config.excludePatterns &&
            new RegExp(this.config.excludePatterns, this.config.excludePatternsOptions).test(operationName)) {
            return "";
        }
        const readString = this.config.generateQueriesRead
            ? `export function readQuery${operationName}(cache: ApolloClient<NormalizedCacheObject>, variables?: ${operationVariablesTypes}):${operationResultType} {
            ${this.config.pre}
            return cache.readQuery({
                query: Operations.${documentVariableName},
                variables,
            });
            ${this.config.post}
            };`
            : "";
        const writeString = this.config.generateQueriesWrite
            ? `export function writeQuery${operationName}(cache: ApolloClient<NormalizedCacheObject>, data: ${operationResultType}, variables?: ${operationVariablesTypes}) {
            ${this.config.pre}
            cache.writeQuery({
                query: Operations.${documentVariableName},
                variables,
                data,
            });

            ${this.config.post}
        }`
            : "";
        return [readString, writeString].filter((a) => a).join("\n");
    }
    buildOperation(node, documentVariableName, operationType, operationResultType, operationVariablesTypes) {
        operationResultType = this._externalImportPrefix + operationResultType;
        operationVariablesTypes =
            this._externalImportPrefix + operationVariablesTypes;
        const cache = this._buildOperationReadCache(node, documentVariableName, operationResultType, operationVariablesTypes);
        return [cache].join("\n");
    }
    buildOperationReadFragmentCache() {
        const res = this._fragments.map((fragment) => {
            const read = this.config.generateFragmentsRead
                ? `export function readFragment${fragment.name}(cache: ApolloClient<NormalizedCacheObject>, fragmentIdProps?: Partial<Types.${fragment.name}Fragment>) {
                ${this.config.pre}
                const dataId = ${this.config.dataIdFromObjectName}({
                  __typename: '${fragment.onType}',
                  ...(fragmentIdProps ? fragmentIdProps : {})
                })
                return cache.readFragment<Types.${fragment.name}Fragment>({
                    id: dataId,
                    fragment: Operations.${fragment.name}FragmentDoc,
                    fragmentName: '${fragment.name}',
                });
                ${this.config.post}
            };`
                : "";
            const write = this.config.generateFragmentsWrite
                ? `export function writeFragment${fragment.name}(cache: ApolloClient<NormalizedCacheObject>, fragmentIdProps?: Partial<Types.${fragment.name}Fragment>, data: Partial<Types.${fragment.name}Fragment>) {
              ${this.config.pre}
              const dataId = ${this.config.dataIdFromObjectName}({
                  __typename: '${fragment.onType}',
                  ...(fragmentIdProps ? fragmentIdProps : {})
              })
              cache.writeFragment({
                id: dataId,
                fragment: Operations.${fragment.name}FragmentDoc,
                data,
              })
              ${this.config.post}
            };
            `
                : "";
            return [read, write].filter((a) => a).join("\n");
        });
        return res.filter((a) => a).join("\n");
    }
}
exports.ApolloCacheVisitor = ApolloCacheVisitor;
