"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const testing_1 = require("@graphql-codegen/testing");
const index_1 = require("../src/index");
const graphql_1 = require("graphql");
const plugin_helpers_1 = require("@graphql-codegen/plugin-helpers");
const typescript_1 = require("@graphql-codegen/typescript");
const typescript_operations_1 = require("@graphql-codegen/typescript-operations");
describe("Apollo Cache", () => {
    let spyConsoleError;
    beforeEach(() => {
        spyConsoleError = jest.spyOn(console, "warn");
        spyConsoleError.mockImplementation();
    });
    afterEach(() => {
        spyConsoleError.mockRestore();
    });
    const schema = graphql_1.buildClientSchema(require("./schema.json"));
    const basicDoc = graphql_1.parse(/* GraphQL */ `
    query test {
      feed {
        id
        commentCount
        repository {
          full_name
          html_url
          owner {
            avatar_url
          }
        }
      }
    }
  `);
    const validateTypeScript = (output, testSchema, documents, config, playground = false) => __awaiter(void 0, void 0, void 0, function* () {
        const tsOutput = yield typescript_1.plugin(testSchema, documents, config, {
            outputFile: "",
        });
        const tsDocumentsOutput = yield typescript_operations_1.plugin(testSchema, documents, config, { outputFile: "" });
        const merged = plugin_helpers_1.mergeOutputs([tsOutput, tsDocumentsOutput, output]);
        testing_1.validateTs(merged, undefined, true, false, playground);
        return merged;
    });
    describe("Imports", () => {
        it("should import apollo-cache-inmemory and apollo-client dependencies", () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = [{ location: "", document: basicDoc }];
            const content = (yield index_1.plugin(schema, docs, {}, {
                outputFile: "graphql.tsx",
            }));
            expect(content.prepend).toContain("import gql from 'graphql-tag';");
            expect(content.prepend).toContain("import { defaultDataIdFromObject, NormalizedCacheObject } from 'apollo-cache-inmemory';");
            expect(content.prepend).toContain("import { ApolloClient } from 'apollo-client';");
            yield validateTypeScript(content, schema, docs, {});
        }));
        it("should import custom apollo-cache-inmemory and apollo-client dependencies", () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = [{ location: "", document: basicDoc }];
            const content = (yield index_1.plugin(schema, docs, {
                apolloCacheImportFrom: "cache",
                apolloImportFrom: "client",
            }, {
                outputFile: "graphql.tsx",
            }));
            expect(content.prepend).toContain("import gql from 'graphql-tag';");
            expect(content.prepend).toContain("import { defaultDataIdFromObject, NormalizedCacheObject } from 'cache';");
            expect(content.prepend).toContain("import { ApolloClient } from 'client';");
            yield validateTypeScript(content, schema, docs, {});
        }));
        it("should import custom  apollo-client v3 dependencies", () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = [{ location: "", document: basicDoc }];
            const content = (yield index_1.plugin(schema, docs, {
                reactApolloVersion: 3,
            }, {
                outputFile: "graphql.tsx",
            }));
            expect(content.prepend).toContain("import { ApolloClient } from '@apollo/client';");
            yield validateTypeScript(content, schema, docs, {});
        }));
        it("should import DocumentNode when using noGraphQLTag", () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = [{ location: "", document: basicDoc }];
            const content = (yield index_1.plugin(schema, docs, {
                noGraphQLTag: true,
            }, {
                outputFile: "graphql.tsx",
            }));
            expect(content.prepend).toContain(`import { DocumentNode } from 'graphql';`);
            expect(content.prepend).not.toContain(`import gql from 'graphql-tag';`);
            yield validateTypeScript(content, schema, docs, {});
        }));
        it(`should use gql import from gqlImport config option`, () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = [{ location: "", document: basicDoc }];
            const content = (yield index_1.plugin(schema, docs, { gqlImport: "graphql.macro#gql" }, {
                outputFile: "graphql.tsx",
            }));
            expect(content.prepend).toContain(`import { gql } from 'graphql.macro';`);
            yield validateTypeScript(content, schema, docs, {});
        }));
    });
    describe("ReadQuery", () => {
        it("Should generate ReadQuery and WriteQuery for query", () => __awaiter(void 0, void 0, void 0, function* () {
            const documents = graphql_1.parse(/* GraphQL */ `
        query feed {
          feed {
            id
            commentCount
            repository {
              full_name
              html_url
              owner {
                avatar_url
              }
            }
          }
        }

        mutation submitRepository($name: String) {
          submitRepository(repoFullName: $name) {
            id
          }
        }
      `);
            const docs = [{ location: "", document: documents }];
            const content = (yield index_1.plugin(schema, docs, {}, {
                outputFile: "graphql.tsx",
            }));
            expect(content.content).toBeSimilarStringTo(`
export function readQueryFeed(cache: ApolloClient<NormalizedCacheObject>, variables?: FeedQueryVariables):FeedQuery {
                 return cache.readQuery({
                    query: Operations.FeedDocument,
                     variables,
                 });
                 };`);
            expect(content.content).toBeSimilarStringTo(`
export function writeQueryFeed(cache: ApolloClient<NormalizedCacheObject>, data: FeedQuery, variables?: FeedQueryVariables) {
                 cache.writeQuery({
                     query: Operations.FeedDocument,
                     variables,
                     data,
                 });
             }`);
            yield validateTypeScript(content, schema, docs, {});
        }));
        it("Should exclude query patterns", () => __awaiter(void 0, void 0, void 0, function* () {
            const documents = graphql_1.parse(/* GraphQL */ `
        query feed {
          feed {
            id
            commentCount
            repository {
              full_name
              html_url
              owner {
                avatar_url
              }
            }
          }
        }

        mutation submitRepository($name: String) {
          submitRepository(repoFullName: $name) {
            id
          }
        }
      `);
            const docs = [{ location: "", document: documents }];
            const content = (yield index_1.plugin(schema, docs, { excludePatterns: "feed", excludePatternsOptions: "i" }, {
                outputFile: "graphql.tsx",
            }));
            expect(content.content).not.toBeSimilarStringTo(`
export function readQueryFeed(cache: ApolloClient<NormalizedCacheObject>, variables?: FeedQueryVariables):FeedQuery {
                 return cache.readQuery({
                    query: Operations.FeedDocument,
                     variables,
                 });
                 };`);
            expect(content.content).not.toBeSimilarStringTo(`
export function writeQueryFeed(cache: ApolloClient<NormalizedCacheObject>, data: FeedQuery, variables?: FeedQueryVariables) {
                 cache.writeQuery({
                     query: Operations.FeedDocument,
                     variables,
                     data,
                 });
             }`);
            yield validateTypeScript(content, schema, docs, {});
        }));
        it("Should not add typesPrefix to hooks", () => __awaiter(void 0, void 0, void 0, function* () {
            const docs = [{ location: "", document: basicDoc }];
            const content = (yield index_1.plugin(schema, docs, { typesPrefix: "I" }, {
                outputFile: "graphql.tsx",
            }));
            expect(content.content).toContain(`export function readQueryTest`);
            expect(content.content).toContain(`export function writeQueryTest`);
        }));
    });
});
